class DebitMac(object):
    transaction_code = None
    transmission_number = None
    bank_response_code = None
    mac_key = None
    pin_key = None
    field_key = None
    trace_number = None
    message_authentication_code = None
