class HostedPaymentData(object):
    """
    Data collection to supplement a hosted payment page.
    """

    customer_exists = None
    customer_key = None
    customer_number = None
    offer_to_save_card = None
    payment_key = None
    product_id = None
    supplementary_data = {}
