class TransactionSummary(object):
    """
    Transaction-level report data
    """
    amount = None
    convenience_amount = None
    shipping_amount = None
    auth_code = None
    authorized_amount = None
    client_transaction_id = None
    device_id = None
    issuer_response_code = None
    issuer_response_message = None
    masked_card_number = None
    original_transaction_id = None
    gateway_response_code = None
    gateway_response_message = None
    reference_number = None
    service_name = None
    settlement_amount = None
    status = None
    transaction_date = None
    transaction_id = None
